==> eh-0.4.tar.gz: eh-0.4/src/ehchkovr.c <==
/***

ehchkovr.c -- Error Handler, check for memory overwrites

Copyright (c) 1992-6 Christopher Cobb, All rights reserved.

***/

static const char *rcsid =
       "$Id: ehchkovr.c,v 1.11 1996/04/26 19:54:10 ccobb Exp $";

#include "ehenv.h"

EHBEGINEXTERNC
#include <string.h>
EHENDEXTERNC

#include "ehmemtrk.h"
#include "ehbounds.h"
#include "ehintres.h"
#include "ehchkovr.h"
#include "ehercode.h"

FILENAME("ehchkovr.c");   /* define current source file name */

/***
ehChkOvr()
Check for memory overwrites.
This function takes advantage of
intimate knowledge of the resource list
and the AVL tree structure.
***/

void
ehChkOvr A4(
   const char *, pcFile,
   const char *, pcFunc,
   const int,    iLine,
   EHRESTRK*, pEhResTrk) /* structure to track this resource */
{
   int iRetVal = 0;
   EHRESCALL *pEhResCall;

   // puts("ehChkOvr: entering");

   if (!pEhResTrk->isInitialized)
   {
      // puts("ehChkOvr: leaving: not initialized");
      return;
   }

   EHLIST *pEhList = &(pEhResTrk->EhList);
   if (pEhList->pEhNodeTailPred == (EHNODE *)pEhList)
   {
      // puts("ehChkOvr: leaving: list is empty");
      return;
   }

   //printf("pEhResTrk->isInitialized %d\n",  pEhResTrk->isInitialized);
   //printf("pEhResTrk->szResCount    %d\n",  pEhResTrk->szResCount);
   //printf("pEhResTrk->ulResObtained %ld\n", pEhResTrk->ulResObtained);
   //printf("pEhResTrk->ulResReleased %ld\n", pEhResTrk->ulResReleased);
   //printf("pEhResTrk->ulResPeak     %ld\n", pEhResTrk->ulResPeak);

   /* iterate through all resource allocation calls */
   for (pEhResCall = (EHRESCALL *)ehListHead(&(pEhResTrk->EhList));
         !ehListEnd((EHNODE*)pEhResCall);
         pEhResCall = (EHRESCALL *)ehListNext((EHNODE*)pEhResCall))
   {
      EHRESLIST *pRes = pEhResCall->pEhResList; /* pick up parent */

      //puts("ehChkOvr: processing a resource call");
      //printf("pEhResCall          = 0x%p\n", pEhResCall);
      //printf("pEhResCall->pcFileO  = %s\n", pEhResCall->pcFileO);
      //if (pEhResCall->pcFuncO)
      //   printf("pEhResCall->pcFuncO  = %s\n", pEhResCall->pcFuncO);
      //printf("pEhResCall->iLineO   = %d\n", pEhResCall->iLineO);
      //if (pEhResCall->pcFileR)
      //   printf("pEhResCall->pcFileR  = %s\n", pEhResCall->pcFileR);
      //if (pEhResCall->pcFuncR)
      //   printf("pEhResCall->pcFuncR  = %s\n", pEhResCall->pcFuncR);
      //printf("pEhResCall->iLineR   = %d\n", pEhResCall->iLineR);
      //puts("");
      //printf("pRes        = 0x%p\n", pRes);
      if (pRes == 0)
      {
         ehCallStack(EHSTDERR);
         break;
      }
//    printf("pRes->ulResHndl =   %lx\n", pRes->ulResHndl);
//    if (pRes->pcResNam)
//       printf("pcResNam %s\n", pRes->pcResNam);
      //printf("pRes->iType =   %d\n", pRes->iType);


      /* we're only interested in memory usage */
      if (pRes->iType != EH_RES_HEAPMEM)
         continue;

      // puts("ehChkOvr: found a memory call");

      /* is this resource still in use */
      if (pRes->ulResHndl &&
            (pRes->EhResState == EH_RES_OBTAINED /*||pRes->szRCInUse*/))
      {
         if (pRes->szRCInUse) /* if it is */
         {

            //puts("ehChkOvr: memory is in use");

            /*** handle resource here ***/

            /* check for buffer overwriting */

//printf("pEhResCall             = 0x%p\n", pEhResCall);
//printf("pEhResCall->pulArgs    = 0x%p\n", pEhResCall->pulArgs);
//printf("pEhResCall->pulArgs[0] = 0x%p\n", pEhResCall->pulArgs[0]);
//printf("pEhResCall->pulArgs[1] = %ld\n",  pEhResCall->pulArgs[1]);

            void * pvMem = (void*)pEhResCall->pulArgs[0];
            size_t szMem = pEhResCall->pulArgs[1];

            //printf("ehChkOvr: pvMem = 0x%p\n", pvMem);
            //printf("ehChkOvr: szMem = %d\n",   szMem);
            // puts("ehChkOvr: about to call ehBndChk");

            if (!pRes->hasProblem && 
                  ehBndChk((char*)pvMem + szMem, EH_SZBOUNDBLK))
            {
               // puts("ehChkOvr: about to call ehHexDmp");

               fprintf(EHSTDERR, 
"          ! ! ! ! !   BUFFER OVERWRITE DETECTED   ! ! ! ! !\n"
                     );
               fprintf(EHSTDERR, 
"          |        --- contents of user buffer ---        |\n"
                     );
               fflush(EHSTDERR);
               ehHexDmp(EHSTDERR, (char*)pvMem, (int)szMem);
               fprintf(EHSTDERR, 
"          |        last sixteen bytes of user buffer ---> |\n"
                     );
               fflush(EHSTDERR);
               ehHexDmp(EHSTDERR,
                     ((char *)pvMem + szMem - 16), 16);
               fprintf(EHSTDERR, 
"          | <--- next 16 bytes after user buffer          |\n"
"          | <--- should cycle thru values 88 99 AA BB     |\n"
                     );
               ehHexDmp(EHSTDERR,
                     ((char *)pvMem + szMem), EH_SZBOUNDBLK);

               /*** make problem known ***/

               ehIndent(EHSTDERR);
               fprintf(EHSTDERR, "memory allocated @: %s %s%s%u\n",
                     pEhResCall->pcFileO, 
                     pEhResCall->pcFuncO?pEhResCall->pcFuncO:"",
                     pEhResCall->pcFuncO?"() ":"",
                     pEhResCall->iLineO);
               fflush(EHSTDERR);
               ehIndent(EHSTDERR);
               fprintf(EHSTDERR, "problem detected @: %s %s%s",
                     pcFile, 
                     pcFunc?pcFunc:"",
                     pcFunc?"() ":"");
               if (iLine) fprintf(EHSTDERR, "%u", iLine);
               fprintf(EHSTDERR, "\n");
               fflush(EHSTDERR);
               ehCallStack(EHSTDERR);

               pRes->hasProblem = true;
               if (Eh.Er.hCode >= 0) Eh.Er = ehUnHash(EH_CTRLC_HASH);
               EhSt.yChain = 1; /* commence chaining */
            }

            /*** end of resource handling ***/

            // puts("ehChkOvr: done processing this memory call");

         } /* end if in use */
      } /* end if OBTAINED or InUse */
   } /* end for pEhResCall */

   //puts("ehChkOvr: leaving");

   return;
}

==> eh-0.5.tar.gz: eh-0.5/src/ehchkovr.c <==
/***

ehchkovr.c -- Error Handler, check for memory overwrites

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char *rcsid =
       "$Id: ehchkovr.c,v 1.11 1996/04/26 19:54:10 ccobb Exp $";

#include "ehchkovr.h"

EHBEGINEXTERNC
#include <string.h>
EHENDEXTERNC

#include "ehmemtrk.h"
#include "ehbounds.h"
#include "ehintres.h"
#include "ehercode.h"

EHFILENAME("ehchkovr.c");   /* define current source file name */

/***
ehChkOvr()
Check for memory overwrites.
This function takes advantage of
intimate knowledge of the resource list
and the AVL tree structure.
***/

void
ehChkOvr A4(
   const char *, pcFile,
   const char *, pcFunc,
   const int,    iLine,
   EHRESTRK*, pEhResTrk) /* structure to track this resource */
{
   const size_t szNumDisp = 16 * 4; /* # rows of 16 bytes displayed */
   int iRetVal = 0;
   EHRESCALL *pEhResCall;

   // puts("ehChkOvr: entering");

   if (!pEhResTrk->isInitialized)
   {
      // puts("ehChkOvr: leaving: not initialized");
      return;
   }

   EHLIST *pEhList = &(pEhResTrk->EhList);
   if (pEhList->pEhNodeTailPred == (EHNODE *)pEhList)
   {
      // puts("ehChkOvr: leaving: list is empty");
      return;
   }

   //printf("pEhResTrk->isInitialized %d\n",  pEhResTrk->isInitialized);
   //printf("pEhResTrk->szResCount    %d\n",  pEhResTrk->szResCount);
   //printf("pEhResTrk->ulResObtained %ld\n", pEhResTrk->ulResObtained);
   //printf("pEhResTrk->ulResReleased %ld\n", pEhResTrk->ulResReleased);
   //printf("pEhResTrk->ulResPeak     %ld\n", pEhResTrk->ulResPeak);

   /* iterate through all resource allocation calls */
   for (pEhResCall = (EHRESCALL *)ehListHead(&(pEhResTrk->EhList));
         !ehListEnd((EHNODE*)pEhResCall);
         pEhResCall = (EHRESCALL *)ehListNext((EHNODE*)pEhResCall))
   {
      EHRESLIST *pRes = pEhResCall->pEhResList; /* pick up parent */

      //puts("ehChkOvr: processing a resource call");
      //printf("pEhResCall          = 0x%p\n", pEhResCall);
      //printf("pEhResCall->pcFileO  = %s\n", pEhResCall->pcFileO);
      //if (pEhResCall->pcFuncO)
      //   printf("pEhResCall->pcFuncO  = %s\n", pEhResCall->pcFuncO);
      //printf("pEhResCall->iLineO   = %d\n", pEhResCall->iLineO);
      //if (pEhResCall->pcFileR)
      //   printf("pEhResCall->pcFileR  = %s\n", pEhResCall->pcFileR);
      //if (pEhResCall->pcFuncR)
      //   printf("pEhResCall->pcFuncR  = %s\n", pEhResCall->pcFuncR);
      //printf("pEhResCall->iLineR   = %d\n", pEhResCall->iLineR);
      //puts("");
      //printf("pRes        = 0x%p\n", pRes);
      if (pRes == 0)
      {
         ehCallStack(EHSTDERR);
         break;
      }
//    printf("pRes->ulResHndl =   %lx\n", pRes->ulResHndl);
//    if (pRes->pcResNam)
//       printf("pcResNam %s\n", pRes->pcResNam);
      //printf("pRes->iType =   %d\n", pRes->iType);


      /* we're only interested in memory usage */
      if ( (pRes->iType != EH_RES_C_MEM) && (pRes->iType != EH_RES_CPP_MEM) )
         continue;

      // puts("ehChkOvr: found a memory call");

      /* is this resource still in use */
      if (pRes->ulResHndl &&
            (pRes->EhResState == EH_RES_OBTAINED /*||pRes->szRCInUse*/))
      {
         if (pRes->szRCInUse) /* if it is */
         {

            //puts("ehChkOvr: memory is in use");

            /*** handle resource here ***/

            /* check for buffer overwriting */

//printf("pEhResCall             = 0x%p\n", pEhResCall);
//printf("pEhResCall->pulArgs    = 0x%p\n", pEhResCall->pulArgs);
//printf("pEhResCall->pulArgs[0] = 0x%p\n", pEhResCall->pulArgs[0]);
//printf("pEhResCall->pulArgs[1] = %ld\n",  pEhResCall->pulArgs[1]);

            void * pvMem = (void*)pEhResCall->pulArgs[0];
            size_t szMem = pEhResCall->pulArgs[1];

            //printf("ehChkOvr: pvMem = 0x%p\n", pvMem);
            //printf("ehChkOvr: szMem = %d\n",   szMem);
            // puts("ehChkOvr: about to call ehBndChk");

            if (!pRes->hasProblem && 
                  ehBndChk((char*)pvMem + szMem, EH_SZBOUNDBLK))
            {
               char *pcOverwrite =
"          ! ! ! ! !   BUFFER OVERWRITE DETECTED   ! ! ! ! !\n";
               char *pcAllocatedAt =
                     "memory allocated @: %s %s%s%d pass %lu\n";
               char *pcProblemAt =
                     "problem detected @: %s %s%s";

               fprintf(EHSTDERR, pcOverwrite);
               if (EHSTDOUT != EHSTDERR) fprintf(EHSTDOUT, pcOverwrite);

               fprintf(EHSTDERR, 
"          |        --- contents of user buffer ---        |\n"
                     );
               fflush(EHSTDERR);
               ehHexDmp(EHSTDERR, (char*)pvMem, 
                     MIN((int)szMem, szNumDisp));
               if (szMem > szNumDisp)
                  fprintf(EHSTDERR, 
"           ... .remainder of % 4d bytes not displayed. ...\n",
                     szMem - szNumDisp);
               fprintf(EHSTDERR, 
"          |        last sixteen bytes of user buffer ---> |\n"
                     );
               fflush(EHSTDERR);
               ehHexDmp(EHSTDERR,
                     ((char *)pvMem + szMem - 16), 16);
               fprintf(EHSTDERR, 
"          | <--- next 16 bytes after user buffer          |\n"
"          | <--- should cycle thru values 88 99 AA BB     |\n"
                     );
               ehHexDmp(EHSTDERR,
                     ((char *)pvMem + szMem), EH_SZBOUNDBLK);

               /*** make problem known ***/

               ehIndent(EHSTDERR);
               fprintf(EHSTDERR, pcAllocatedAt,
                     pEhResCall->pcFileO, 
                     pEhResCall->pcFuncO?pEhResCall->pcFuncO:"",
                     pEhResCall->pcFuncO?"() ":"",
                     abs(pEhResCall->iLineO),
                     pEhResCall->ulCountO);
               fflush(EHSTDERR);
               if (EHSTDOUT != EHSTDERR)
               {
                  ehIndent(EHSTDOUT);
                  fprintf(EHSTDOUT, pcAllocatedAt,
                        pEhResCall->pcFileO, 
                        pEhResCall->pcFuncO?pEhResCall->pcFuncO:"",
                        pEhResCall->pcFuncO?"() ":"",
                        abs(pEhResCall->iLineO),
                        pEhResCall->ulCountO);
                  fflush(EHSTDOUT);
               }
               ehIndent(EHSTDERR);
               fprintf(EHSTDERR, pcProblemAt,
                     pcFile, 
                     pcFunc?pcFunc:"",
                     pcFunc?"() ":"");
               if (iLine) fprintf(EHSTDERR, "%d", abs(iLine));
               fprintf(EHSTDERR, " pass %lu\n", pEhResCall->ulCountO);
               fflush(EHSTDERR);
               if (EHSTDOUT != EHSTDERR)
               {
                  ehIndent(EHSTDOUT);
                  fprintf(EHSTDOUT, pcProblemAt,
                        pcFile, 
                        pcFunc?pcFunc:"",
                        pcFunc?"() ":"");
                  if (iLine) fprintf(EHSTDOUT, "%d", abs(iLine));
                  fprintf(EHSTDOUT," pass %lu\n",pEhResCall->ulCountO);
                  fflush(EHSTDOUT);
               }
               ehCallStack(EHSTDERR);

               pRes->hasProblem = true;
               if (Eh.Er.hCode >= 0) Eh.Er = ehUnHash(EH_CTRLC_HASH);
               EhSt.yChain = 1; /* commence chaining */
            }

            /*** end of resource handling ***/

            // puts("ehChkOvr: done processing this memory call");

         } /* end if in use */
      } /* end if OBTAINED or InUse */
   } /* end for pEhResCall */

   //puts("ehChkOvr: leaving");

   return;
}
